;xxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxx
; Nom du projet : Tutorial
; Auteur : Fabrice SINCERE
; Date : 02/06
; Microcontrleur : Microchip PIC 16F84A
; Langage : Assembleur
; IDE : Microchip MPLAB
;xxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxx

;xxxxxxxxxxx
; Processeur
;xxxxxxxxxxx

	List p=16F84A
	#include <p16F84A.inc>

;xxxxxxxxxxxxxxxxxxxxxx
; Bits de configuration
;xxxxxxxxxxxxxxxxxxxxxx

	__config _CP_OFF & _WDT_OFF & _PWRTE_ON & _RC_OSC

	;code protect OFF
	;watchdog timer OFF
	;power up timer ON
	;oscillateur RC

;xxxxxx
; Macro
;xxxxxx

bank1	macro	; passage en banque 1
	bsf STATUS,RP0
	endm

bank0	macro	; passage en banque 0
	bcf STATUS,RP0
	endm

;xxxxxxxxxxxxxxxxxxxxxxxxxxx
; Initialisation de l'EEPROM
;xxxxxxxxxxxxxxxxxxxxxxxxxxx

	org 0x2100	; adresse absolue du dbut de l'EEPROM
	DE 0x50	; adresse 0x00
	DE 0x69	; adresse 0x01
	DE 0x63	; adresse 0x02
	DE 0x69	; adresse 0x03
	DE 0x6E	; adresse 0x04
	DE 0x69	; adresse 0x05
	DE 0x74	; adresse 0x06
	DE 0x00	; adresse 0x07
	DE 0x00	; adresse 0x08
	DE 0x00	; adresse 0x09
	DE 0x00	; adresse 0x0A
	DE 0x00	; adresse 0x0B
	DE 0x00	; adresse 0x0C
	DE 0x00	; adresse 0x0D
	DE 0x00	; adresse 0x0E
	DE 0x00	; adresse 0x0F
	DE 0x00	; adresse 0x10
	DE 0x00	; adresse 0x11
	DE 0x00	; adresse 0x12
	DE 0x00	; adresse 0x13
	DE 0x00	; adresse 0x14
	DE 0x00	; adresse 0x15
	DE 0x00	; adresse 0x16
	DE 0x00	; adresse 0x17
	DE 0x00	; adresse 0x18
	DE 0x00	; adresse 0x19
	DE 0x00	; adresse 0x1A
	DE 0x00	; adresse 0x1B
	DE 0x00	; adresse 0x1C
	DE 0x00	; adresse 0x1D
	DE 0x00	; adresse 0x1E
	DE 0x00	; adresse 0x1F
	DE 0x00	; adresse 0x20
	DE 0x00	; adresse 0x21
	DE 0x00	; adresse 0x22
	DE 0x00	; adresse 0x23
	DE 0x00	; adresse 0x24
	DE 0x00	; adresse 0x25
	DE 0x00	; adresse 0x26
	DE 0x00	; adresse 0x27
	DE 0x00	; adresse 0x28
	DE 0x00	; adresse 0x29
	DE 0x00	; adresse 0x2A
	DE 0x00	; adresse 0x2B
	DE 0x00	; adresse 0x2C
	DE 0x00	; adresse 0x2D
	DE 0x00	; adresse 0x2E
	DE 0x00	; adresse 0x2F
	DE 0x00	; adresse 0x30
	DE 0x00	; adresse 0x31
	DE 0x00	; adresse 0x32
	DE 0x00	; adresse 0x33
	DE 0x00	; adresse 0x34
	DE 0x00	; adresse 0x35
	DE 0x00	; adresse 0x36
	DE 0x00	; adresse 0x37
	DE 0x00	; adresse 0x38
	DE 0x00	; adresse 0x39
	DE 0x00	; adresse 0x3A
	DE 0x00	; adresse 0x3B
	DE 0x00	; adresse 0x3C
	DE 0x00	; adresse 0x3D
	DE 0x00	; adresse 0x3E
	DE 0xAA	; adresse 0x3F
	
;xxxxxxxxxxxxxxxxxxxxxxxxxxx
; Dclaration des variables
;xxxxxxxxxxxxxxxxxxxxxxxxxxx

	CBLOCK 0x0C

	W_TEMP : 1 	; adresse 0x0C
	STATUS_TEMP : 1 	; adresse 0x0D
	count_1 : 1 	; adresse 0x0E
	count_2 : 1 	; adresse 0x0F
	count_3 : 1 	; adresse 0x10
	index_1 : 1 	; adresse 0x11
	index_2 : 1 	; adresse 0x12
	index_3 : 1 	; adresse 0x13
	
	ENDC

;xxxxxxxxxxxxxxxxxxxx
; Dmarrage sur reset
;xxxxxxxxxxxxxxxxxxxx

	org 0x0000
	goto initialisation

;xxxxxxxxxxxxxxxxxxxxxxx
; Routine d'interruption
;xxxxxxxxxxxxxxxxxxxxxxx

	org 0x0004	; vecteur d'interruption
	movwf W_TEMP	; sauvegarde des registres W et STATUS
	swapf STATUS,W
	movwf STATUS_TEMP

	; Routine d'interruption

	swapf STATUS_TEMP,W	; restauration des registres STATUS et W
	movwf STATUS
	swapf W_TEMP,f
	swapf W_TEMP,W
	retfie	; retour d'interruption

;xxxxxxxxxxxxxxx
; Initialisation
;xxxxxxxxxxxxxxx

initialisation

	bank0
	clrf PORTA
	clrf PORTB

	bank1

	movlw B'00010100'
	movwf OPTION_REG
	; bit 7 (/RBPU) = 0 : activation des rsistances de pull-up du port B
	; bit 6 (INTEDG) = 0 : interruption RB0/INT sur front descendant
	; bit 5 (T0CS) = 0 : l'horloge interne est l'horloge du timer0
	; bit 4 (T0SE) = 1 : valeur par dfaut (option non utilise)

	; bit 3 (PSA) = 0
	; bit 2 (PS2) = 1
	; bit 1 (PS1) = 0
	; bit 0 (PS0) = 0
	; Prescaler du Timer0 = 1:32
	; Watchdog sans prescaler (1:1)

	movlw B'00000111'
	movwf TRISA
	; bit 7 du port A : non implment
	; bit 6 du port A : non implment
	; bit 5 du port A : non implment
	; bit 4 du port A (RA4/T0CKI) = 0 : configuration en sortie
	; bit 3 du port A (RA3) = 0 : configuration en sortie
	; bit 2 du port A (RA2) = 1 : configuration en entre
	; bit 1 du port A (RA1) = 1 : broche non connecte (par dfaut, configuration en entre)
	; bit 0 du port A (RA0) = 1 : configuration en entre
	
	movlw B'00111001'
	movwf TRISB
	; bit 7 du port B (RB7) = 0 : configuration en sortie
	; bit 6 du port B (RB6) = 0 : configuration en sortie
	; bit 5 du port B (RB5) = 1 : configuration en entre
	; bit 4 du port B (RB4) = 1 : configuration en entre
	; bit 3 du port B (RB3) = 1 : broche non connecte (par dfaut, configuration en entre)
	; bit 2 du port B (RB2) = 0 : configuration en sortie
	; bit 1 du port B (RB1) = 0 : configuration en sortie
	; bit 0 du port B (RB0/INT) = 1 : configuration en entre

	bank0

	movlw B'00001000'
	movwf PORTA
	; sortie RA4/T0CKI du port A au niveau 0
	; sortie RA3 du port A au niveau 1
	
	movlw B'10000100'
	movwf PORTB
	; sortie RB7 du port B au niveau 1
	; sortie RB6 du port B au niveau 0
	; sortie RB2 du port B au niveau 1
	; sortie RB1 du port B au niveau 0
	
	clrf count_1
	movlw 0x0E
	movwf count_3
	movlw 0xF3
	movwf index_1
	movlw 0xF4
	movwf index_2
	clrf index_3
	
	clrf TMR0

	movlw B'10110000'
	movwf INTCON
	; bit 7 (GIE) = 1 : autorisation globale des interruptions
	; bit 6 (EEIE) = 0 : interdiction de l'interruption de fin d'criture de l'EEPROM
	; bit 5 (T0IE) = 1 : autorisation de l'interruption de dbordement de TMR0
	; bit 4 (INTE) = 1 : autorisation de l'interruption RB0/INT
	; bit 3 (RBIE) = 0 : interdiction de l'interruption RB
	; bit 2 (T0IF) = 0 : effacement du drapeau de l'interruption TMR0
	; bit 1 (INTF) = 0 : effacement du drapeau de l'interruption RB0/INT
	; bit 0 (RBIF) = 0 : effacement du drapeau de l'interruption RB

	goto debut_programme

;xxxxxxxxxxxxxxxxxxxx
; Programme principal
;xxxxxxxxxxxxxxxxxxxx

debut_programme

	; Programme principal

	goto debut_programme

	END